# -*- coding: utf-8 -*-


import os
import re
import sys
from clasaua.lib.clasaua import Clasaua


def clasaua():
    print("Launcher clasaua function")
    arguments = sys.argv
    app_file_path = re.sub(r'(-script\.py|-script\.pyw|\.exe)?$', '', arguments[0])
    if len(arguments) > 1:
        file_path = arguments[1]
        if os.path.isfile(file_path):
            file_path = file_path
            app_path_folder = os.path.dirname(os.path.realpath(__file__))
            print("app_path_folder: {}".format(app_path_folder))        
            work_path_folder = os.getcwd()
            # check logos in work_path_folder
            images = (
                'logo_foot.png',
                'logo_left.png',
                'logo_right.png')
            images_in_work_path_folder = True
            for image in images:
                image_path = '{}{}{}'.format(work_path_folder, os.sep, image) 
                print("image_path: {}".format(image_path))
                if not os.path.isfile(image_path):
                    images_in_work_path_folder = False
            if images_in_work_path_folder:
                images_in_work_path_folder = work_path_folder
            print("work path folder: {}".format(work_path_folder))
            Clasaua(app_path_folder=app_path_folder, file_path=file_path, work_path_folder=images_in_work_path_folder)
        else:
            print("Clasifications file path not exists.")
    else:
        print("Please specify the ODS file with the classification.")


if __name__ == "__main__":
    print("Launcher main")
    sys.exit(clasaua())



# -*- coding: utf-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Copyright (C) 2019 Federacion Galega de Natación (FEGAN) http://www.fegan.org
# Author: Daniel Muñiz Fontoira (2017) <dani@damufo.com>


import os
import gzip


def get_file_content(file_path, mode="all", compressed=True, encoding=None):
    '''
    open and read file
    mode = [all:read|lines:readlines]
    '''
    content = ""

    if os.path.isfile(file_path):
        if compressed:
            fil = gzip.open(file_path, 'rb')  # , encoding=encoding
            if mode == "all":
                if encoding:
                    content = fil.read().decode(encoding)
                else:
                    content = fil.read().decode()
            else:
                if encoding:
                    content = [i.decode(encoding) for i in fil.readlines()]
                else:
                    content = [i.decode() for i in fil.readlines()]
        else:
            fil = open(file_path, 'r', encoding=encoding)
            if mode == "all":
                content = fil.read()
            else:
                content = fil.readlines()
        fil.close()
        content = content  # .decode('utf-8')
    return content


def set_file_content(content, file_path, compress=True, binary=False,
                     encoding=None, end_line="\n"):
    '''
    write content to file. Content can be string or list/tuple strings
    binary (only for no compress), when true 'wb' when false 'w'
    compress: compress gzip
    encoding, example "ISO-8859-1"
    end_line: [\n | \r\n]
    '''

    if compress or binary:
        if compress:
            fil = gzip.open(file_path, 'wb', encoding=encoding)
        else:
            fil = open(file_path, 'wb', encoding=encoding)

        if isinstance(content, (list, tuple)):
            for i in content:
                fil.write("{}{}".format(i, end_line).encode())
        elif isinstance(content, str):
            fil.write(content.encode())

    else:
        fil = open(file_path, 'w', encoding=encoding)

        if isinstance(content, (list, tuple)):
            for i in content:
                fil.write("{}{}".format(i, end_line))
        elif isinstance(content, str):
            fil.write(content)
    fil.close()


def parse_line(line, separator='#'):
    '''
    line: unicode string
    return unicode fields
    '''
    value = line.strip()
    value = value.replace('[/n]', "\n")
#     value = str(value, 'utf-8').split(separator) python2
    value = value.split(separator)

    return value


def parse_lines(lines, separator='#'):
    '''
    line: unicode string
    return unicode list of fields
    '''
    lines_parsed = []
    for i in lines:
        lines_parsed.append(parse_line(line=i, separator=separator))
    return lines_parsed


def export_table_to_file(dbs, tables, file_path, where_sql='', where_values=()):
    '''
    export to file to share
    tables is a tuple of tables
    '''

    content = ""
    for table_name in tables:
        sql = "select * from {} {}"
        sql = sql.format(table_name, where_sql)
        res = dbs.exec_sql(sql=sql, values=(where_values,))
        if res:
            content += "#**#%s\n" % (table_name)
            for i in res:
                line = ''
                for n in i:
                    campo = str(n)
                    campo = campo.replace("#", "-")
                    line = "%s%s#" % (line, campo)
                line = "%s" % (line[:(len(line)-1)],)
                line = line.replace('\n', "[/n]")  # Engadido para evitar
                # retornos de carro nos ficheiros intermedios
                line = line.replace('\u2018', ",")
                line = line.replace('\u2019', ",")
                line = line.replace('\u201c', '"')
                line = line.replace('\u201d', '"')
                line = line.replace("False", "0")
                line = line.replace("True",  "1")
                line = line.replace("None", "")
                content += "%s\n" % line
            print("Feito %s" % table_name)
    content = content

    set_file_content(content=content, file_path=file_path, compress=True)



# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Copyright (C) 2019 Federacion Galega de Natación (FEGAN) http://www.fegan.org
# Author: Daniel Muñiz Fontoira (2017) <dani@damufo.com>

import os
import sys
import re
from operator import attrgetter

import gettext
from gettext import ngettext
import locale 

from odf.opendocument import OpenDocumentText, load
from odf.table import Table, TableRow, TableCell
from odf import text

from reportlab.lib.units import mm

from clasaua.lib.report_base import ReportBase
from clasaua.lib.files import get_file_content
from clasaua.lib.clubs import clubs

# ARGS = sys.argv[0] = re.sub(r'(-script\.py|-script\.pyw|\.exe)?$', '', sys.argv[0])
# print(ARGS)

APP_VERSION =  '0.4.10'
APP_VERSION_DATE =  '2024-09-02'


EVENTS = open('events.csv', 'r')
EVENTS = EVENTS.read().splitlines()
EVENTS = tuple(EVENTS)

# EVENTS = (
#     'Cto. Galego de Augas Abertas',
#     'Travesía do Caneiro',
# )

# EVENTS_CHOICES = tuple([(i, i) for i in EVENTS]) 

# EVENTS = (
#     "Cto. Galego de Augas Abertas",
#     "Travesía do Caneiro",
#     "Travesía ao Dique",
#     "Travesía Fluvial do Lérez",
#     "Travesía Vila do Tea",
#     "Travesía Vilagarcía de Arousa",
#     "Travesía Illa de Bensa",
#     "Travesía Descenso do Río Miño",
#     "Travesía de Miño",
#     "Travesía Castrelo de Miño",
# )

ETAPA_FINAL = 9 # OLLO! o valor de event_id é un menos que o número da travesia
PUNTOS_EXTRA = 7  # número de probas con puntos para obter puntos extra
NUMERO_MINIMO_CLASIFICAR = 4
NUMERO_MAXIMO_SUMAR = 5

COUNT_EVENTS = len(EVENTS)
# Puntuations
PUN_MASTE = (15, 12, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1)
PUN_DEPOR = (30, 25, 21, 18, 16, 15, 14, 13, 12,
            11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1)

PUN_MASTE_FINAL = (20, 16, 13, 11, 10, 9, 8, 7, 6, 5, 4, 3)
PUN_DEPOR_FINAL = (42, 35, 29, 24, 20, 17, 15, 14, 13,
            12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2)


class Result():

    def __init__(self, event_id, pos, category_points):
        """
        category = ['ELITE'|'MASTE']
        """
        self.event_id = event_id
        self.pos = pos
        if event_id == ETAPA_FINAL:  # OLLO! o valor de event_id é un menos que o número da travesia
            if category_points == 'MASTE':
                list_points = PUN_MASTE_FINAL
            else:
                list_points = PUN_DEPOR_FINAL

            if pos >= len(list_points):
                points = 0
            else:
                points = list_points[pos]
        else:
            if category_points == 'MASTE':
                list_points = PUN_MASTE
            else:
                list_points = PUN_DEPOR
            if pos >= len(list_points):
                points = 0
            else:
                points = list_points[pos]
        self.points = points

    @property
    def name(self):
        return EVENTS[self.pos]


class Person():
    def __init__(self, person_id, full_name, gender_id, category_id, club_id):
        self.person_id = person_id
        self.name = full_name.split(',')[1].strip()
        self.surname = full_name.split(',')[0].strip()

        self.gender_id = gender_id
        self.category_id = category_id
        self.club_id = club_id
        self.results = {}

    def add_result(self, event_id, pos, category_points):
        self.results[event_id] = Result(event_id, pos, category_points)
    
    @property
    def club_name(self):
        club_name = ''
        if self.club_id in clubs:
            club_name = clubs[self.club_id][0]
        return club_name

    @property
    def results_text(self):
        values = []
        for event_id in range(COUNT_EVENTS):
            if event_id in self.results:
                values.append(str(self.results[event_id].points))
            else:
                values.append('')
        values = '#'.join(values)
        return values

    @property
    def total_points(self):
        total = -1
        if len(self.results) >= NUMERO_MINIMO_CLASIFICAR:
            total = 0
            results_sorted = sorted(
                self.results.values(), key=attrgetter('points'),
                reverse=True)
            for item in results_sorted[:NUMERO_MAXIMO_SUMAR]:
                total += item.points
        # Só se contan probas con puntos
        results_with_points = len([ i for i in self.results.values() if i.points > 0])
        if len(self.results) != results_with_points:
            print("ten resultados sen puntos")
        if results_with_points >= PUNTOS_EXTRA:
            if self.category_id == 'ELITE':
                total += 20
            else:
                total += 10
        return total

# class GrowingList(list): USADO POR VERSION marcoconti83 PODE BORRARSE
#     def __setitem__(self, index, value):
#         if index >= len(self):
#             self.extend([None]*(index + 1 - len(self)))
#         list.__setitem__(self, index, value)


class Clasaua():
    """
    Xerador de clasificacións do Circuíto Galego de Augas Abertas
    """

    def __init__(self, app_path_folder, file_path, work_path_folder):
        """
        file_path is the ods clasifications file
        """
        language = 'gl'
        if not language:
            language = 'en'
        # locale.getlocale()[0][0:2]
        filename = "locale/messages_{}.mo".format(language)

        try:
            trans = gettext.GNUTranslations(open(filename, "rb"))
        except IOError:
            trans = gettext.NullTranslations()

        trans.install() 

        self.app_path_folder = app_path_folder
        self.work_path_folder = work_path_folder
        self.app_version = APP_VERSION
        self.app_version_date = APP_VERSION_DATE
        self.file_path = file_path
        self.persons = {}
        # print(arguments)
        # if len(arguments) > 1:
        #     print(arguments)
        #     file_path = arguments[1]
        #     if os.path.isfile(file_path):
                # self.file_path = file_path
        #         self.app_path_folder = os.path.dirname(os.path.realpath(__file__))            
        #         self.work_dir = os.getcwd()
        #         print(self.app_path_folder)
        #         print(self.work_dir)
        self.get_data_ods()
        self.gen_person_report()
        self.gen_club_report()
        #     else:
        #         print(_("Clasifications file path not exists."))
        # else:
        #     print(_("Please specify the ODS file with the classification."))



    def get_data_ods(self):
        '''
        Obtén os datos directamente do ODS.
        OLLO! cando unha cela está repetida o ODS márcaa como repetida e pon
        unha columna menos.
        Tamén se podería ter en conta o caso de celas xuntadas. Neste caso
        non e contempla. 
        '''

        spreadsheetdoc = load(self.file_path)

        d = spreadsheetdoc.spreadsheet
        rows = d.getElementsByType(TableRow)
        lines = []
        for row in rows:
            cells = row.getElementsByType(TableCell)

            line = []
            for cell in cells:
                repeateds = cell.getAttribute("numbercolumnsrepeated")
                if repeateds and int(repeateds) > 1:
                # cell = cell.getElementsByType(text.P)
                    for repeated in range(int(repeateds)):
                        if cell.__str__():
                            line.append(cell.__str__())
                else:
                    if cell.__str__():
                        line.append(cell.__str__())

            if len(line) == 7:
                lines.append(line)
            else:
                print("Columnas de máis.")

        (EVENT_ID, POS, PERSON_ID, CLUB_ID, FULL_NAME, GENDER_ID,
         CATEGORY_ID) = range(7)
        persons = {}

        for values in lines:
            event_id = int(values[EVENT_ID]) - 1
            pos = int(values[POS]) - 1
            person_id = values[PERSON_ID].strip()
            club_id = values[CLUB_ID].strip()
            if len(club_id) < 5:
                club_id = club_id.zfill(5)
            full_name = values[FULL_NAME].strip()
            gender_id = values[GENDER_ID].strip()
            category_id = values[CATEGORY_ID].strip()
            if category_id == 'ABSO':
                category_id = 'ELITE'
            if person_id in persons:
                person = persons[person_id]
            else:
                person = Person(
                    person_id, full_name, gender_id, category_id, club_id)
                persons[person_id] = person
            category_points = {
                'MASTER1': 'MASTE',
                'MASTER2': 'MASTE',
                'MASTER3': 'MASTE',
                'MASTER4': 'MASTE',
                'ELITE': 'ELITE',
            }
            person.add_result(event_id, pos, category_points[category_id])
        self.persons = persons

    # def get_data_csv_non_se_usa_pode_borrarse(self):
    #     '''
    #     Usado na versión inicial
    #     '''
    #     content = get_file_content(file_path=self.file_path,
    #                                mode="lines",
    #                                compressed=False,
    #                                encoding="utf8")

    #     (EVENT_ID, POS, PERSON_ID, CLUB_ID, FULL_NAME, GENDER_ID,
    #      CATEGORY_ID) = range(7)
    #     persons = {}

    #     for line in content:
    #         values = line.strip().split('#')
    #         event_id = int(values[EVENT_ID]) - 1
    #         pos = int(values[POS]) - 1
    #         person_id = values[PERSON_ID]
    #         club_id = values[CLUB_ID]
    #         if len(club_id) < 5:
    #             club_id = club_id.zfill(5)
    #         full_name = values[FULL_NAME]
    #         gender_id = values[GENDER_ID]
    #         category_id = values[CATEGORY_ID]
    #         if person_id in persons:
    #             person = persons[person_id]
    #         else:
    #             person = Person(
    #                 person_id, full_name, gender_id, category_id, club_id)
    #             persons[person_id] = person
    #         person.add_result(event_id, pos)
    #     self.persons = persons

    def gen_person_report(self):

        persons_list = self.persons.values()
        persons_sorted = sorted(
            persons_list, key=attrgetter('surname', 'name'),
            reverse=False)
        persons_sorted = sorted(
            persons_sorted, key=attrgetter('total_points'), reverse=True)

        persons_sorted = sorted(
            persons_sorted, key=attrgetter('category_id', 'gender_id'),
            reverse=False)

        # for item in persons_sorted:
        #     line = '{} {} {} {} {}, {} {} {}'.format(
        #         item.category_id, item.gender_id, item.total_points,
        #         item.person_id, item.surname, item.name, item.club_name,
        #         item.results_text)

        # file_path = os.path.join(self.app_path_folder, 'clas_depor.pdf')
        file_path = 'clas_depor.pdf'
        d = ReportBase(
            app_path_folder=self.app_path_folder,
            app_version=self.app_version,
            file_path=file_path,
            orientation='portrait',
            title="Circuíto Galego de Augas Abertas",
            subtitle='Tempada 2023/24',
            work_path_folder=self.work_path_folder)

        d.insert_paragraph(
            "<b>Clasificacións individuais por categoría</b>", "CENTER")
        d.insert_spacer(1, 12)
        NUM_EVENTS = len(EVENTS)
        # Isto pode cambiar cada tempada
        cabeceira = ('Pos', 'Licenza', 'Apelidos', 'Nome', 'Club')
        cabeceira += tuple(str(val+1) for val in range(NUM_EVENTS))
        cabeceira += ('Tot.', )
        table = []
        lines_title = []
        category_id = None
        gender_id = None
        num_line = 0
        pos = 1
        pos_equated = 0
        last_points = 0
        for item in persons_sorted:

            if item.category_id != category_id or item.gender_id != gender_id:
                gender = {'F': 'Feminina', 'M': 'Masculina'}
                line = [('Categoría {} {}'.format(item.category_id,
                                                  gender[item.gender_id]))]
                lines_title.append(num_line)
                table.append(line)
                table.append(cabeceira)
                category_id = item.category_id
                gender_id = item.gender_id
                num_line += 2
                pos = 0
                last_points = 0
                pos_equated = 0
            if item.total_points != -1:
                total_points = item.total_points
                if last_points:
                    if last_points == item.total_points:
                        pos_line = str(pos)
                        pos_equated += 1
                    else:
                        pos += pos_equated
                        pos_equated = 0
                        pos += 1
                        pos_line = str(pos)
                        last_points = item.total_points
                else:
                    pos += pos_equated
                    pos_equated = 0
                    pos += 1
                    pos_line = str(pos)
                    last_points = item.total_points
            else:
                pos_line = ''
                last_points = -1
                total_points = ''

            line = [pos_line, item.person_id, item.surname,
                    item.name, item.club_name]
            sep_events = item.results_text.split('#')
            line.extend(sep_events)
            line.extend([total_points])
            table.append(line)
            num_line += 1

        style = [
            ('FONTSIZE', (0, 0), (-1, -1), 6),
            ('GRID', [ 0, 0 ], [ -1, -1 ], 0.05, 'grey' ),
            ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
            ('ALIGN',(2, 0),(3, -1), 'LEFT'),
            # ('VALIGN', (0, 0), (-1, -1), 'TOP')
            ('TOPPADDING', (0,0), (-1, -1), 3),
            # ('LEFTPADDING', (0,0), (-1,-1), 8),
            # ('RIGHTPADDING', (0,0), (-1,-1), 3),
            ('BOTTOMPADDING', (0, 0), (-1, -1), -2),
            ]
        for i in lines_title:
            # style.append(('ALIGN',(0,i),(-1,-1), 'LEFT'))
            style.append(('FONT', (0, i), (-1, i), 'Open Sans Bold'))
            style.append(('SPAN', (0, i), (-1, i)))  # clube
            style.append(('FONT', (0, i+1), (-1, i+1), 'Open Sans Bold'))
            # style.append(('SPAN', (0, i+1), (-1, i+1)))  # clube

        col_widths = ['3%', '7%', '20%', '12%', '11%']

        col_widths.extend(['{}%'.format((42/NUM_EVENTS)),] * NUM_EVENTS)
        col_widths.extend(['5%',])
        # row_heghts = [14*mm]
        row_heghts = None
        d.insert_table(table=table, colWidths=col_widths,
                       rowHeights=row_heghts,
                       style=style, pagebreak=False)

        d.insert_spacer(1, 24)
        table = []
        table.append(('Relación de sedes:', ))
        total_items = NUM_EVENTS
        corte = total_items//3
        if total_items % 3:
            corte += 1
        for i in range(corte):
            line = []
            if i < total_items:
                line.append('{}. {}'.format(i+1, EVENTS[i]))
            if (i + corte) < total_items:
                line.append('{}. {}'.format(i+corte+1, EVENTS[i+corte]))
            else:
                line.append('')
            if (i + (corte*2)) < total_items:
                line.append('{}. {}'.format(i+(corte*2)+1, EVENTS[i+(corte*2)]))
            else:
                line.append('')


            table.append(line)
        style = [
            ('FONTSIZE', (0, 0), (-1, -1), 6),
            ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
        ]

        col_widths = ['33%', '33%', '34%']
        # row_heghts = [4*mm]*15
        row_heghts = None
        table = d.formata_taboa(table, font_size=7)
        d.insert_table(table=table, colWidths=col_widths,
                       rowHeights=row_heghts,
                       style=style, pagebreak=False)

        d.build_file()

    def gen_club_report(self):
        punt_by_club = 2  # Number of participants by category by club
        puntuations = []
        persons_list = self.persons.values()
        for license_id, person in self.persons.items():
            for event_id, event in person.results.items():
                puntuations.append((
                    license_id, person.club_id, person.gender_id,
                    person.category_id, event_id, event.points))

        (LICENSE, CLUB, GENDER, CATEGORY, EVENT, POINTS) = range(6)
        puntuations = sorted(
            puntuations,
            key=lambda puntuation: puntuation[POINTS],
            reverse=True)
        puntuations = sorted(
            puntuations,
            key=lambda puntuation: (
                puntuation[CLUB], puntuation[GENDER], puntuation[CATEGORY],
                puntuation[EVENT]),
            reverse=False)
        count_club = 0
        current_club = None
        clas_fem_depor = {}
        clas_mas_depor = {}
        clas_fem_maste = {}
        clas_mas_maste = {}
        # Converte as puntuacións de diccionario a táboa
        count_club = 0
        current_club = None
        (LICENSE, CLUB, GENDER, CATEGORY, EVENT, POINTS) = range(6)
        for i in puntuations:
            club = '{}.{}.{}.{}'.format(
                i[CATEGORY], i[GENDER], i[EVENT], i[CLUB])
            if current_club != club:
                current_club = '{}.{}.{}.{}'.format(
                    i[CATEGORY], i[GENDER], i[EVENT], i[CLUB])
                count_club = 0

            if count_club < punt_by_club:
                if i[GENDER] == 'F':
                    if i[CATEGORY] == 'ELITE':
                        if not i[CLUB] in clas_fem_depor:
                            clas_fem_depor[i[CLUB]] = 0
                        clas_fem_depor[i[CLUB]] += i[POINTS]
                    else:
                        if not i[CLUB] in clas_fem_maste:
                            clas_fem_maste[i[CLUB]] = 0
                        clas_fem_maste[i[CLUB]] += i[POINTS]
                elif i[GENDER] == 'M':
                    if i[CATEGORY] == 'ELITE':
                        if not i[CLUB] in clas_mas_depor:
                            clas_mas_depor[i[CLUB]] = 0
                        clas_mas_depor[i[CLUB]] += i[POINTS]
                    else:
                        if not i[CLUB] in clas_mas_maste:
                            clas_mas_maste[i[CLUB]] = 0
                        clas_mas_maste[i[CLUB]] += i[POINTS]
                count_club += 1

        # convert to tuple
        clas_fem_depor = [ (k, v) for k, v  in clas_fem_depor.items()]
        clas_mas_depor = [ (k, v) for k, v  in clas_mas_depor.items()]
        clas_fem_maste = [ (k, v) for k, v  in clas_fem_maste.items()]
        clas_mas_maste = [ (k, v) for k, v  in clas_mas_maste.items()]
        # sort
        clas_fem_depor = sorted(clas_fem_depor, key=lambda tup: tup[1],
                                reverse=True)
        clas_mas_depor = sorted(clas_mas_depor, key=lambda tup: tup[1],
                                reverse=True)
        clas_fem_maste = sorted(clas_fem_maste, key=lambda tup: tup[1],
                                reverse=True)
        clas_mas_maste = sorted(clas_mas_maste, key=lambda tup: tup[1],
                                reverse=True)

        # file_path = os.path.join(self.app_path_folder, 'clas_club.pdf')
        file_path = 'clas_club.pdf'

        d = ReportBase(
            app_path_folder=self.app_path_folder,
            app_version=self.app_version,
            file_path=file_path,
            orientation='portrait',
            title="Circuíto Galego de Augas Abertas",
            subtitle='Tempada 2023/24',
            work_path_folder=self.work_path_folder)
        d.insert_spacer(1, 12)

        clasifications = (
            ('Clasificación por clubs elite feminina', clas_fem_depor),
            ('Clasificación por clubs elite masculina', clas_mas_depor),
            ('Clasificación por clubs máster feminina', clas_fem_maste),
            ('Clasificación por clubs máster masculina', clas_mas_maste),
        )
        for clasification in clasifications:
            title = clasification[0]
            res = clasification[1]
            # d.insert_paragraph("<b>{}</b>".format(title), "LEFT")
            d.insert_title_1("{}".format(title), 0)
            d.insert_spacer(1, 10)
            table = []
            current_points = None
            current_pos = 0
            for i in res:
                club_id = i[0]
                club_name = ''
                if club_id in clubs:
                    club_name = clubs[club_id][1]
                points = i[1]
                if current_points != points:
                    current_pos += 1
                    current_points = points
                    position = current_pos
                else:
                    position = ''

                line = (position, club_name, points, '')
                table.append(line)

            col_widths = ['5%', '40%', '15%', '30%']
            # col_widths = ['3%', '10%', '7%', '13%', '10%','67%']
            # row_heghts = [14*mm]
            row_heghts = None

            style = [
                        ('FONTSIZE', (0, 0), (-1, -1), 10),
                        ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
                        ('ALIGN',(2, 0),(2, -1), 'RIGHT'),
                        ('TOPPADDING', (0,0), (-1, -1), 3),
                        ('BOTTOMPADDING', (0, 0), (-1, -1), -2),
                    ]
            d.insert_table(
                table=table,
                colWidths=col_widths,
                rowHeights=row_heghts,
                style=style,
                pagebreak=False,
                alignment='CENTER',
                )
            d.insert_spacer(1, 30)


        d.build_file()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# version 20220614

from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer
from reportlab.platypus import Table
from reportlab.platypus import PageBreak
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.units import cm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase.pdfmetrics import registerFontFamily
from reportlab.lib.pagesizes import A4, landscape, portrait
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY
from reportlab.lib import colors
from PIL import Image
import os
import copy
from datetime import datetime


class ReportBase(object):

    def __init__(self, app_path_folder, app_version, file_path, orientation='portrait', title='',
                 subtitle='', work_path_folder=None):
        '''
        Constructor
        orientation= [portrait|landscape] (vertical|horizontal)
        '''
        self.timestamp = datetime.now().strftime("%Y%m%d%H%M%S") 
        self.colors = colors
        self.orientation = orientation
        if orientation == 'portrait': #vertical
            self.page_height = 29.7*cm
            self.page_width = 21*cm
            pagesize = portrait(A4)
        elif orientation == 'landscape': #landscape
            self.page_height = 21*cm
            self.page_width = 29.7*cm
            pagesize = landscape(A4)
        self.app_path_folder = app_path_folder
        if work_path_folder:
            images_path = '%s%s' % (work_path_folder, os.sep) 
        else:
            images_path = '%s%s%s%s' % (self.app_path_folder, os.sep, 
                                    'images', os.sep)
        self.app_version = app_version
        self.author = os.environ.get('USER', os.environ.get('USERNAME'))
        self.creator = 'Clasaua {}'.format(self.app_version)
        self.title = title
        self.subtitle = subtitle
        
        self.logo_left = Image.open('%s%s' % (images_path, 'logo_left.png'))
        self.logo_right = Image.open('%s%s' % (images_path, 'logo_right.png'))
        self.logo_foot = Image.open('%s%s' % (images_path, 'logo_foot.png'))

        self.estiloencabezado = ParagraphStyle('',
                                      fontName='Open Sans Bold',
                                      fontSize=8,
                                      #FIXME: fontStyle='small-cups', isto non funciona
                                      alignment=TA_CENTER,
                                      spaceBefore=0.5*cm,
                                      spaceAfter=0*cm,
                                      leftIndent=-1*cm,
                                      rightIndent=-0.7*cm)
        self.estiloencabezado2 = ParagraphStyle('',
                                      fontName='Open Sans Bold',
                                      fontSize=6,
                                      #FIXME: fontStyle='small-cups', isto non funciona
                                      alignment=TA_CENTER,
                                      spaceBefore=0.5*cm,
                                      spaceAfter=0*cm,
                                      leftIndent=-1*cm,
                                      rightIndent=-0.7*cm)         
        self.style_normal = ParagraphStyle('normal',
                                           fontName='Open Sans Regular',
                                           fontSize=10,
                                           alignment=TA_JUSTIFY,
                                           spaceBefore=0*cm,
                                           spaceAfter=0.2*cm,
                                           firstLineIndent=0*cm,
#                                         topIndent =-1*cm,
#                                         leftIndent=-1*cm,
#                                         rightIndent=-0.7*cm
                                           )
        self.style_normal_right = ParagraphStyle('normal_right',
                                      parent=self.style_normal,
                                      alignment=TA_RIGHT
                                      )
        self.style_normal_left = ParagraphStyle('normal_left',
                                      parent=self.style_normal,
                                      alignment=TA_LEFT
                                      )
        self.style_normal_center = ParagraphStyle('normal_center',
                                      parent=self.style_normal,
                                      alignment=TA_CENTER
                                      )
        self.style_normal_justify = ParagraphStyle('normal_justify',
                                      parent=self.style_normal,
                                      alignment=TA_JUSTIFY
                                      )
        self.contido_taboa = ParagraphStyle('normal',
                                            fontName='Open Sans Regular',
#                                            fontSize=9,
#                                            alignment=TA_JUSTIFY,
#                                            spaceBefore=0*cm,
#                                            spaceAfter=0.2*cm,
#                                            firstLineIndent=0*cm,
#                                         topIndent =-1*cm,
#                                         leftIndent=-1*cm,
#                                         rightIndent=-0.7*cm
                                           )
        self.titulo1 = ParagraphStyle('',
                                      fontName='Open Sans Regular',
                                      fontSize=12,
                                      alignment=TA_CENTER,
                                      spaceBefore=0.5*cm,
                                      spaceAfter=0*cm,
        #                              leftIndent=-1*cm,
        #                              rightIndent=-0.7*cm
                                      )

        self.t1_table = ParagraphStyle('',
                                      fontName='Open Sans Regular',
                                      fontSize=12,
                                      alignment=TA_CENTER,
                                      spaceBefore=1.5*cm,
                                      spaceAfter=0.5*cm,
        #                              leftIndent=-1*cm,
        #                              rightIndent=-0.7*cm
                                      )

        self.titulo2 = ParagraphStyle('',
                                      fontName='Open Sans Regular',
                                      fontSize=10,
                                      alignment=TA_CENTER,
                                      spaceBefore=0.5*cm,
                                      spaceAfter=0*cm,
        #                              leftIndent=-1*cm,
        #                              rightIndent=-0.7*cm
                                      )
        self.titulo1_center = copy.copy(self.titulo1)
        self.titulo1_center.alignment = 1

        self.titulo1_left = copy.copy(self.titulo1)
        self.titulo1_left.alignment = 0

        self.t2_table = ParagraphStyle('',
                                      fontName='Open Sans Regular',
                                      fontSize=10,
                                      alignment=TA_CENTER,
                                      spaceBefore=5.5*cm,
                                      spaceAfter=0.5*cm,
                                      topIndent=3*cm,
        #                              leftIndent=-1*cm,
        #                              rightIndent=-0.7*cm
                                      )
        self.titulo3 = ParagraphStyle('',
                                      fontName='Open Sans Bold',
                                      fontSize=10,
                                      alignment=TA_CENTER,
                                      spaceBefore=0.5*cm,
                                      spaceAfter=0*cm,
                                      leftIndent=-1*cm,
                                      rightIndent=-0.7*cm)
        #import TT font
        fonts_path = '%s%s%s%s' % (self.app_path_folder, os.sep, 
                                    'fonts', os.sep)
        pdfmetrics.registerFont(TTFont('Open Sans Regular', '%s%s%s%s' % (
                fonts_path,"open-sans", os.sep, "OpenSans-Regular.ttf")))
        pdfmetrics.registerFont(TTFont('Open Sans Bold', '%s%s%s%s' % (
                fonts_path,"open-sans", os.sep, 'OpenSans-Bold.ttf')))
        pdfmetrics.registerFont(TTFont('Open Sans Italic', '%s%s%s%s' % (
                fonts_path,"open-sans", os.sep, 'OpenSans-Italic.ttf')))
        pdfmetrics.registerFont(TTFont('Open Sans Bold Italic', '%s%s%s%s' % (
                fonts_path,"open-sans", os.sep, 'OpenSans-BoldItalic.ttf')))
        registerFontFamily('Open Sans', normal='Open Sans Regular', 
                           bold='Open Sans Bold', 
                           italic='Open Sans Italic', 
                           boldItalic='Open Sans Bold Italic')

        self.doc = SimpleDocTemplate(file_path, pagesize=pagesize,
                        rightMargin=30, leftMargin=30,
                        topMargin=72, bottomMargin=3.2*cm)

#        Este outro casca co informe historico de records ¿?
#        self.doc = SimpleDocTemplate(file_path, pagesize=pagesize,
#                        rightMargin=1.5*cm,leftMargin=1.5*cm,
#                        topMargin=2.5*cm,bottomMargin=1.8*cm)
        self.story = []
#        for i in range(6):
#            self.insert_title_1(u'Capítulo %s: La formación de las virutas de chocolate.' % i)
#            self.insert_paragraph(u'Un texto muy, muy, muy, pero que muuuuuuuy largo. '*20)
#            self.insert_paragraph(u'Un texto muy, muy, muy, pero que muuuuuuuy largo. '*20)
#            self.story.append(Spacer(1,0.2*cm))
            
    def build_file(self):
#        self.doc.build(self.story, onFirstPage=self.my_first_page, onLaterPages=self.my_later_pages)
        self.doc.build(self.story, onFirstPage=self.my_first_page, onLaterPages=self.my_first_page)

    def insert_title_1(self, text, alignment=TA_JUSTIFY):
        '''
        TA_LEFT, TA_CENTER or TA_CENTRE, TA_RIGHT and TA_JUSTIFY, 
        with values of 0, 1, 2 and 4 respectively
        '''
        if alignment == TA_CENTER:
            title = Paragraph(text, self.titulo1_center)
        elif alignment == TA_LEFT:
            title = Paragraph(text, self.titulo1_left)
        else:
            title = Paragraph(text, self.titulo1)
        self.story.append(title)

    def get_t1_table(self, text):
        t1 = Paragraph(text, self.t1_table)
        return t1

    def insert_title_2(self, text, alignment=TA_LEFT):
        '''
        TA_LEFT, TA_CENTER or TA_CENTRE, TA_RIGHT and TA_JUSTIFY,
        with values of 0, 1, 2 and 4 respectively
        '''
        if alignment == TA_CENTER:
            title = Paragraph(text, self.titulo1_center)
        else:
            title = Paragraph(text, self.titulo1)
        self.story.append(title)

    def insert_page_break(self):
        self.story.append(PageBreak())

    def get_t2_table(self, text):
        t2 = Paragraph(text, self.t2_table)
        return t2

    def insert_spacer(self, width, height):
        spacer = Spacer(width, height)
        self.story.append(spacer)

    def insert_paragraph(self, text, align=None):
        if not align:
            style = self.style_normal
        elif align.upper() == "RIGHT":
            style = self.style_normal_right
        elif align.upper() == "LEFT":
            style = self.style_normal_left
        elif align.upper() == "CENTER":
            style = self.style_normal_center
        elif align.upper() == "JUSTIFY":
            style = self.style_normal_justify

        p = Paragraph(text=text, style=style)
        self.story.append(p)

    def formata_taboa(self, table, col_aligns=[], font_size=9):
        style = ParagraphStyle('normal',
                               fontName='Open Sans Regular',
                               fontSize=font_size,
                               )
        table_formated = []
        for i in table:
            line = []
            for j in i:
                paragraph_formatted = Paragraph(text=j,
                                                style=style)
                line.append(paragraph_formatted)
            table_formated.append(line)
        return table_formated

    def insert_table(self, table, colWidths=None, rowHeights=None, style=None, pagebreak=False, alignment=None):
        """
        aligment = [LEFT|RIGHT|CENTER]
        """
        if table:
#        Style([('FONT',(0,0),(-1,-1), 'Helvetica'), 
##                    ('FONTSIZE',(0,0),(-1,-1), 8),
##                    ('TEXTCOLOR',(0,0),(0,2), colors.blue), 
##                    ('TEXTCOLOR', (1,0), (1,2),colors.green)]
#        self.story.append(Spacer(0, prev_spacer))

            if pagebreak:
                self.story.append(PageBreak())
            t = Table(table, colWidths=colWidths, rowHeights=rowHeights, style=style)
            if alignment:
                t.hAlign = alignment
            self.story.append(t)
        
    def my_first_page(self, canvas, doc):
        
        canvas.setAuthor(self.author)
        canvas.setCreator(self.creator)
        canvas.setSubject('')
        canvas.setTitle(self.title)
        
        canvas.saveState()
        start_y_foot = 3.2*cm
        # start_y_foot = 1.5*cm
    ##    linhas
        canvas.setStrokeColor('Grey')
        canvas.setLineWidth(0.01)

        l1 = (1*cm, self.page_height-2.3*cm, self.page_width-1.5*cm, self.page_height-2.3*cm)
        l2 = (1*cm, start_y_foot, self.page_width-1*cm, start_y_foot)
#        self.lineas = [l1,l2]
        # self.lineas = [l2] #só liña de embaixo
        # self.lineas = []  # ningunha
        # canvas.lines(self.lineas)
    ##    Textos
        canvas.setFont('Open Sans Regular',7)
    ##    cabeceira
        canvas.drawInlineImage(self.logo_left, 1*cm, self.page_height-(2.2*cm), width = 75, height = 47)
        canvas.drawInlineImage(self.logo_right, self.page_width-(4*cm), self.page_height-(2.2*cm), width = 75, height = 47)

        # Centra o pe
        foot_x_pos = ((self.page_width)/2)-(269.291338583)
        # canvas.drawInlineImage(self.logo_foot, 1*cm, start_y_foot-(2.7*cm), width = 1072*0.5, height = 120*0.5)
        if self.orientation == 'landscape':
            print(self.page_width)
            print('landscape')
        else:
            print(self.page_width)
            print('portrait')
        canvas.drawInlineImage(self.logo_foot, foot_x_pos, start_y_foot-(2.7*cm), width = 1072*0.5, height = 120*0.5)

#        titulo da cabeceira
        canvas.setFont('Open Sans Regular', 10)
        canvas.setFillColorRGB(0,0.176,0.447)  # azul escuro
        canvas.setFillColorRGB(0.075, 0.137, 0.357)  # azul escuro
        string_with_total = canvas.stringWidth(self.title)
        canvas.drawString((self.page_width/2)-(string_with_total/2), 
                          self.page_height-(1.5*cm), self.title)
#        subtítulo da cabeceira
        canvas.setFont('Open Sans Regular', 8)
        canvas.setFillColorRGB(0, 0, 0) #black
        string_with_total = canvas.stringWidth(self.subtitle)
        canvas.drawString((self.page_width/2)-(string_with_total/2), 
                          self.page_height-(1.9*cm), self.subtitle)
 
        # DEREITA CABECEIRA
        # canvas.setFont('Open Sans Regular', 8)
        # canvas.setFillColorRGB(0, 0.176, 0.447)  # azul escuro
        # canvas.setFillColorRGB(0.075, 0.137, 0.357)  # azul escuro
        # canvas.drawRightString(self.page_width - (1.25 * cm),
        #                        self.page_height - (0.9 * cm),
        #                        'Federación Galega de Natación')
        # string_with_total = canvas.stringWidth('Federación Galega de Natación')
        # canvas.setFont('Open Sans Regular', 6)
        # canvas.setFillColorRGB(0, 0, 0)  # black
        # string_width = (string_with_total/2) - (canvas.stringWidth('Avenida de Glasgow, 13')/2)
        # canvas.drawRightString((self.page_width-(1.25*cm)-string_width), self.page_height-1.3*cm, 'Avenida de Glasgow, 13')
        # string_width = (string_with_total/2) - (canvas.stringWidth('CP 15008 - A Coruña')/2)
        # canvas.drawRightString((self.page_width-(1.25*cm)-string_width), (self.page_height-1.6*cm), 'CP 15008 - A Coruña')
        # string_width = (string_with_total/2) - (canvas.stringWidth('www.fegan.org - info@fegan.org')/2)
        # canvas.drawRightString((self.page_width-(1.25*cm)-string_width), self.page_height-1.9*cm, 'www.fegan.org - info@fegan.org')

    ##    pe
        # pos_y_foot = 0.6 * cm
        pos_y_text_foot = start_y_foot - 0.35 * cm
        canvas.setFont('Open Sans Regular', 6)
        # Pe esquerda
        canvas.drawString(1 * cm, pos_y_text_foot, _('Clasaua v.{}').format(self.app_version))
        # Pe centro
        pe_centro_texto = _('Timestamp: {}').format(self.timestamp)
        string_with_total = canvas.stringWidth(pe_centro_texto)
        canvas.drawString((self.page_width/2)-(string_with_total/2), 
                          pos_y_text_foot, pe_centro_texto)
        # pe dereita
        canvas.drawRightString(self.page_width - 1 * cm, pos_y_text_foot, _('Page %d') % doc.page)
        
        canvas.restoreState()
        
    def my_later_pages(self, canvas, doc):
        '''
        xa non o uso, emprego o modelo da primeira páxina para todo.
        '''
        canvas.saveState()
    ##    Lineas
        canvas.setStrokeColor('Grey')
        canvas.setLineWidth(0.01)
        canvas.lines(self.lineas)
    ##    Textos
        canvas.setFont('Open Sans Regular',7)
    ##    Cabecera
        canvas.drawInlineImage(self.logo, 1*cm, self.page_height-(2.2*cm), width = 75, height = 47)
        canvas.setFont('Open Sans Bold', 8)
        canvas.setFillColorRGB(0,0.176,0.447) #azul escuro
        string_with_total = canvas.stringWidth('Federación Galega De Natación')
        canvas.drawRightString(self.page_width-1.7*cm, self.page_height-0.9*cm, 'Federación Galega De Natación')
        canvas.setFont('Open Sans Regular', 6)
        canvas.setFillColorRGB(0,0,0) #black
        string_width = (string_with_total/2) - (canvas.stringWidth('Avenida de Glasgow, 13')/2)
        canvas.drawRightString((self.page_width-1.7*cm-string_width), self.page_height-1.3*cm, 'Avenida de Glasgow, 13')
        string_width = (string_with_total/2) - (canvas.stringWidth('CP 15008 - A Coruóa')/2)
        canvas.drawRightString((self.page_width-1.7*cm-string_width), (self.page_height-1.6*cm), 'CP 15008 - A Coruña')
        string_width = (string_with_total/2) - (canvas.stringWidth('www.fegan.org - info@fegan.org')/2)
        canvas.drawRightString((self.page_width-1.7*cm-string_width), self.page_height-1.9*cm, 'www.fegan.org - info@fegan.org')
    ##    Pie
        canvas.drawRightString(self.page_width - 1.7 * cm, 1.0 * cm, _('page %d') % doc.page)
        
        canvas.restoreState()
                




# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Copyright (C) 2019 Federacion Galega de Natación (FEGAN) http://www.fegan.org
# Author: Daniel Muñiz Fontoira (2017) <dani@damufo.com>


"""
xesde sql to update list
select "'" || club_id ||"': ('" || short_desc ||"', '" || long_desc || "'),"
from clubs where not club_id like "D%"order by club_id;
"""


import csv
clubs = {}
with open('clubs.csv', encoding="utf8", newline='') as f:
    reader = csv.reader(f, dialect='unix', delimiter=',')
    for row in reader:
        clubs[row[0]] = (row[1], row[2])
